/* OS/2 REXX: create a list of all EA types found on local disks. */
/* Observations:                                                  */
/* - .TYPE should be MVMT (0xFFDF), but often it's ASCII (0xFFFD) */
/* - .VERSION should be also MVMT, but in a few cases was ASCII   */
/* - whatever .APPTYPE is, it is either BINARY (0xFFFE) or MVMT   */
/* - VENDORNAME (printer, OMNI resp. PostScript) is MVMT or ASCII */
/* - REQUIREDDRIVERFILES (OMNI resp. PostScript) is MVMT or ASCII */
/* - OPTIONALDRIVERFILES (OMNI resp. PostScript) is MVMT or ASCII */
/* - REXX.VARIABLEBUF and REXX.LITERALPOOL dont't have an EA type */
/* - REXX.METACONTROL was often 0xEA60, with a few dubious 0x2B67 */
/* - toolkit 3 EAS.exe uses ASCIIZ (0xFFFC), this is now obsolete */
/* - toolkit 3 EAS.exe uses FILE name (0xFFEF), now also obsolete */
/* - toolkit 2 BSEDOS.H explicitly states that ASCIIZ is obsolete */

   signal on  novalue  name TRAP    ;  signal on  syntax name TRAP
   signal on  failure  name TRAP    ;  signal on  halt   name TRAP
   call UTIL 'SysFileTree'          ;  call UTIL 'SysQueryEAList'
   call UTIL 'SysGetEA'             ;  ISVIO = ( PROC() < 3 )
   ALL.0 = 0                        ;  EXPO = 'ISVIO ALL. EAS.'

   parse source . . TMP             ;  N = lastpos( '\', TMP )
   TMP = substr( TMP, N + 1 )       ;  N = lastpos( '.', TMP )
   TMP = left( TMP, N ) || 'tmp'    ;  N = XENV( 'TMP' )
   if N = '' then N = '.'           ;  TMP = N || '\' || TMP
   call XDEL TMP                    ;  call CTOZ

   if ISVIO then say d2c( 13 ) || left( 'writing' TMP || '...', 79 )

   do N = 1 to ALL.0
      THIS = ALL.N
      THIS = left( THIS || ' ', 29, . ) EAS.THIS
      call lineout TMP, THIS
   end N

   exit 0

CTOZ: procedure expose (EXPO)    /* call SUBS on all local drives */
   call UTIL 'SysDriveMap'          ;  call UTIL 'SysFileSystemType'
   D = SysDriveMap( /**/, 'Local' )

   do N = 1 to words( D )        /* no EA access on CDFS (why ?)  */
      if SysFileSystemType( word( D, N )) = 'CDFS' then  iterate N
      if SUBS(              word( D, N ))          then  return 1
   end N
   return 0

SUBS: procedure expose (EXPO)
   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return FAIL                   /* 1: errors are simply ignored  */

DOIT: procedure expose (EXPO)    /* 1: error for e.g. EA DATA. SF */
   if ISVIO then call charout /**/, x2c( 0D ) || left( arg(1), 79 )

   THIS = SysQueryEAList( arg( 1 ), 'EA' )
   if THIS <> 32 & THIS <> 0 then exit TRAP( 'error' THIS )

   if THIS = 0 then do N = 1 to EA.0
      THIS = SysGetEA( arg( 1 ), EA.N, 'VAL' )
      select
         when THIS = 32                   then VAL = 'locked'
         when THIS <> 0                   then VAL = ' error'
         when EA.N = 'REXX.VARIABLEBUF'   then VAL = '  n/a '
         when EA.N = 'REXX.LITERALPOOL'   then VAL = '  n/a '
         when length( VAL ) < 2           then VAL = ' short'
         otherwise do
            VAL = c2x( reverse( left( VAL, 2 )))
            select
               when VAL = 'FFFE'          then VAL = 'BINARY'
               when VAL = 'FFFD'          then VAL = 'ASCII '
               when VAL = 'FFFC'          then VAL = 'ASCIIZ'
               when VAL = 'FFFB'          then VAL = 'BITMAP'
               when VAL = 'FFFA'          then VAL = 'META  '
               when VAL = 'FFF9'          then VAL = 'ICON  '

               when VAL = 'FFEF'          then VAL = 'FILE  '
               when VAL = 'FFEE'          then VAL = 'EA    '
               when VAL = 'FFDF'          then VAL = 'MVMT  '
               when VAL = 'FFDE'          then VAL = 'MVST  '
               when VAL = 'FFDD'          then VAL = 'ASN1  '

               otherwise   VAL = '0x' || VAL
            end
         end
      end

      EA.N = VAL EA.N   ;  THIS = EA.N
      if symbol( 'EAS.THIS' ) = 'VAR' then do
         if datatype( EAS.THIS, 'w' )
            then EAS.THIS = EAS.THIS + 1
            else EAS.THIS = 2
      end
      else do
         EAS.THIS = arg( 1 )  ;  THIS = ALL.0 + 1
         ALL.0 = THIS         ;  ALL.THIS = EA.N
      end
   end N

   return 0

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

XDEL: procedure                  /* semi-portable SysFileDelete() */
   parse source KEY . .
   if KEY = 'OS/2' then do
      call UTIL 'SysFileDelete'
      return SysFileDelete( arg( 1 )) <> 0   /* 0: okay, 1: error */
   end
   parse version KEY . .
   if KEY = 'REXXSAA'
      then return RxDelete( arg( 1 )) <> 0   /* 0: okay, 1: error */
      else return DosDel(   arg( 1 )) == 0   /* 0: okay, 1: error */

XENV: procedure                  /* DOS REXX portable environment */
   parse version ENV . .
   if ENV = 'REXXSAA' then do
      parse source ENV . .       /* OS/2 REXXSAA:  os2environment */
      if ENV = 'OS/2'   then ENV = 'OS2ENVIRONMENT'
                        else ENV = ENV || 'ENVIRONMENT'
   end                           /* DOS  REXXSAA:  DOSENVIRONMENT */
   else  ENV = 'ENVIRONMENT'     /* REXX/Personal:    environment */
   select
      when arg() = 1 then  return value( arg( 1 ),/* get */, ENV )
      when arg() = 2 then  return value( arg( 1 ), arg( 2 ), ENV )
      otherwise            return abs( /* force REXX error 40 */ )
   end

PROC: procedure                  /* avoid "unknown function" TRAP */
   parse source OS .             /* for REXXSAA portability abuse */
   if OS <> 'OS/2' then return 1 /* the now obsolete 1: real mode */

   OS = 'ProcessType'            /* assume Sys... = RxProcessType */
   if RxFuncQuery( 'Sys' || OS ) = 0 then return SysProcessType()
   if RxFuncAdd(   'Sys' || OS, 'RxUtils', 'Rx' || OS ) = 0 then do
      signal on syntax name PROC.TRAP  ;  return SysProcessType()
   end                           /* tries RxUtils only once, else */
PROC.TRAP:                       /* force RexxUtil SysProcessType */
   call  RxFuncDrop 'SysProcessType'   ;  signal on syntax name TRAP
   call        UTIL 'SysProcessType'   ;  return SysProcessType()

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 0 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
